@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2016 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2017 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -402,7 +402,7 @@ NOEXPORT int getaddrinfo(const char *node, const char *service,
     char *tmpstr;
 
     if(!node)
-        node=hints->ai_flags&AI_PASSIVE ? DEFAULT_ANY : DEFAULT_LOOPBACK;
+        node=(hints->ai_flags & AI_PASSIVE) ? DEFAULT_ANY : DEFAULT_LOOPBACK;
 #if defined(USE_WIN32) && !defined(_WIN32_WCE)
     if(s_getaddrinfo)
         return s_getaddrinfo(node, service, hints, res);
@@ -452,7 +452,7 @@ NOEXPORT int getaddrinfo(const char *node, const char *service,
     /* not numerical: need to call resolver library */
     *res=NULL;
     ai=NULL;
-    CRYPTO_w_lock(stunnel_locks[LOCK_INET]);
+    stunnel_write_lock(&stunnel_locks[LOCK_INET]);
 #ifdef HAVE_GETHOSTBYNAME2
     h=gethostbyname2(node, AF_INET6);
     if(h) /* some IPv6 addresses found */
@@ -470,7 +470,7 @@ NOEXPORT int getaddrinfo(const char *node, const char *service,
 #ifdef HAVE_ENDHOSTENT
     endhostent();
 #endif
-    CRYPTO_w_unlock(stunnel_locks[LOCK_INET]);
+    stunnel_write_unlock(&stunnel_locks[LOCK_INET]);
     if(retval) { /* error: free allocated memory */
         freeaddrinfo(*res);
         *res=NULL;
@@ -605,10 +605,10 @@ int getnameinfo(const struct sockaddr *sa, socklen_t salen,
                 (void *)&((struct sockaddr_in *)sa)->sin_addr,
             host, hostlen);
 #else /* USE_IPv6 */
-        CRYPTO_w_lock(stunnel_locks[LOCK_INET]); /* inet_ntoa is not mt-safe */
+        stunnel_write_lock(&stunnel_locks[LOCK_INET]); /* inet_ntoa is not mt-safe */
         strncpy(host, inet_ntoa(((struct sockaddr_in *)sa)->sin_addr),
             hostlen);
-        CRYPTO_w_unlock(stunnel_locks[LOCK_INET]);
+        stunnel_write_unlock(&stunnel_locks[LOCK_INET]);
         host[hostlen-1]='\0';
 #endif /* USE_IPv6 */
     }
