@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2016 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2017 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -47,9 +47,10 @@ NOEXPORT int compression_init(GLOBAL_OPTIONS *);
 NOEXPORT int prng_init(GLOBAL_OPTIONS *);
 NOEXPORT int add_rand_file(GLOBAL_OPTIONS *, const char *);
 
-int index_cli, index_opt, index_redirect, index_addr;
+int index_ssl_cli, index_ssl_ctx_opt;
+int index_session_authenticated, index_session_connect_address;
 
-int ssl_init(void) { /* init SSL before parsing configuration file */
+int ssl_init(void) { /* init TLS before parsing configuration file */
 #if OPENSSL_VERSION_NUMBER>=0x10100000L
     OPENSSL_init_ssl(OPENSSL_INIT_LOAD_SSL_STRINGS |
         OPENSSL_INIT_LOAD_CRYPTO_STRINGS, NULL);
@@ -57,15 +58,17 @@ int ssl_init(void) { /* init SSL before parsing configuration file */
     SSL_load_error_strings();
     SSL_library_init();
 #endif
-    index_cli=SSL_get_ex_new_index(0, "cli index",
-        NULL, NULL, NULL);
-    index_opt=SSL_CTX_get_ex_new_index(0, "opt index",
-        NULL, NULL, NULL);
-    index_redirect=SSL_SESSION_get_ex_new_index(0, "redirect index",
-        NULL, NULL, NULL);
-    index_addr=SSL_SESSION_get_ex_new_index(0, "addr index",
-        NULL, NULL, cb_free);
-    if(index_cli<0 || index_opt<0 || index_redirect<0 || index_addr<0) {
+    index_ssl_cli=SSL_get_ex_new_index(0,
+        "CLI pointer", NULL, NULL, NULL);
+    index_ssl_ctx_opt=SSL_CTX_get_ex_new_index(0,
+        "SERVICE_OPTIONS pointer", NULL, NULL, NULL);
+    index_session_authenticated=SSL_SESSION_get_ex_new_index(0,
+        "session authenticated", NULL, NULL, NULL);
+    index_session_connect_address=SSL_SESSION_get_ex_new_index(0,
+        "session connect address", NULL, NULL, cb_free);
+    if(index_ssl_cli<0 || index_ssl_ctx_opt<0 ||
+            index_session_authenticated<0 ||
+            index_session_connect_address<0) {
         s_log(LOG_ERR, "Application specific data initialization failed");
         return 1;
     }
@@ -113,7 +116,7 @@ NOEXPORT void cb_free(void *parent, void *ptr, CRYPTO_EX_DATA *ad,
     str_free(ptr);
 }
 
-int ssl_configure(GLOBAL_OPTIONS *global) { /* configure global SSL settings */
+int ssl_configure(GLOBAL_OPTIONS *global) { /* configure global TLS settings */
 #ifdef USE_FIPS
     if(FIPS_mode()!=global->option.fips) {
         RAND_set_rand_method(NULL); /* reset RAND methods */
